#ifndef _MY_SERIAL_H
#define _MY_SERIAL_H

#include <windows.h>
#include <string>
//#include "/include/c++/3.4.5/string"

class serial_port
{
public:
   serial_port();
	serial_port(char* portname);
	~serial_port();

	int open();
	int open(char* portname);
	void close();

   // Configure baud rate and character format
   //
   // baud:
   //   Supply one of the supported rates. In your code, assign the integer baud rate,
   //   or the corresponding enum code - see MSDN documentation 
   //   (or the documentation of your serial driver)
   // char_len:
   //   Supply the character length, i.e. the number of payload data bits: 8,7,6 or 5. 
   // parity_enable:
   //    TRUE  = parity generation & checking = ON
   //    FALSE = parity generation & checking = OFF
   // parity_type:
   //    NOPARITY | ODDPARITY | EVENPARITY | MARKPARITY | SPACEPARITY
   // stop_bits:
   //    ONESTOPBIT | ONE5STOPBITS | TWOSTOPBITS
   //
   // returns: 0 = success; anything else = failure
   //
	int cfg_baud_and_format(
         unsigned long baud,
         unsigned char char_len,
         unsigned long parity_enable,
         unsigned char parity_type,
         unsigned char stop_bits
                          );

   // Congigure HW flow control
   // 
   // rts (= output at the PC): 
   //    RTS_CONTROL_DISABLE   = RTS inactive (can be toggled)
   //    RTS_CONTROL_ENABLE    = RTS active (can be toggled)
   //    RTS_CONTROL_HANDSHAKE = RTS involved in HW flow control (follows RX FIFO tresholds)
   //    RTS_CONTROL_TOGGLE    = RTS follows "UART *TX* FIFO not empty" (for RS485?)
   //    
   // cts (= input at the PC):
   //    FALSE = ignore CTS, keep sending
   //    TRUE  = CTS involved in HW flow control (TX is blocked while CTS is inactive)
   //    
   // returns: 0 = success; anything else = failure
   //
   int cfg_hw_flowctl(unsigned int rts, unsigned int cts);
   
   // Configure SW flow control
   // 
   // x_in:
   //    FALSE = don't expect XON/XOFF in RX data
   //    TRUE  = do expect and do observe XON/XOFF in RX data
   //
   // x_out:
   //    FALSE = disable transmission of XON/XOFF
   //    TRUE  = enable transmission of XON/XOFF
   //    
   // returns: 0 = success; anything else = failure
   //
   int cfg_sw_flowctl(unsigned int x_out, unsigned int x_in);

   // Configure call control via DTR+DSR
   // 
   // dtr (= output at the PC): 
   //    DTR_CONTROL_DISABLE   = DTR inactive (can be toggled)
   //    DTR_CONTROL_ENABLE    = DTR active (can be toggled)
   //    DTR_CONTROL_HANDSHAKE = DTR involved in HW call control (COM open() => DTR active)
   //    
   // dsr (= input at the PC):
   //    FALSE = ignore DSR, keep sending
   //    TRUE  = DSR involved in HW flow/call control (TX is blocked while DSR is inactive)
   //
   // Note that with modems, DTR is perhaps complementary to CD rather than DSR.
   // And, CD seems to be completely out of band, has to be followed explicitly.
   // Modems steer the DSR signal "according to V.25". Difficult to say exactly
   // what that means... Essentially, DSR seems to be up as soon as the modem
   // is ready to accept commands - i.e., even before dialing, let alone successful
   // carrier negotiation. The one occasion when DSR might be dropped by the modem,
   // is when the modem detects an off-hook condition on a parallel telephone set.
   //
   // returns: 0 = success; anything else = failure
   //
   int cfg_callctl(unsigned int dtr, unsigned int dsr);

   // Set COM port timeouts (milliseconds)
   // 
   // This stuff is heavily specific to the Microsoft Windows driver
   // (a purely software affair).
   // See the MSDN documentation for a full description of the complete
   // semantics of these timeouts.
   // 
   // returns: 0 = success; anything else = failure
   //
	int set_comm_timeouts(
         unsigned long ReadIntervalTimeout,
         unsigned long ReadTotalTimeoutMultiplier,
         unsigned long ReadTotalTimeoutConstant,
         unsigned long WriteTotalTimeoutMultiplier,
         unsigned long WriteTotalTimeoutConstant
                        );

   // returns: 0 = success; anything else = failure
	int read_char(unsigned char &data);

   // returns: 0 = success; anything else = failure
	int write_char(unsigned char data);

   int read(unsigned char* data, unsigned long len); // returns the number of bytes read
   int write(unsigned char* data, unsigned long len); // returns the number of bytes written

   // expects a null-terminated string
   // returns 0 if OK, returns 1 if an error occurred
   int write_string(char* data);

   // look for a terminating <CR> == ^M == \r
   //
   // Implemented using read_char(): simplistic but functional.
   // The timeout is a per-character timeout in milliseconds.
   //
   // returns:
   //    0 = OK, newline detected, see data in buffer
   //    1 = read_char() failed (timeout, device error...)
   //    2 = buffer full, no newline encountered
   // Note: invalidates previous timeout settings of the COM port
   //
   int read_line(char* data, unsigned long buf_len, unsigned long timeout);

   // Check the incoming data for a set of strings, 
   // specified as a NULL-terminated array of zero-terminated char*'s . 
   // When you find an occurrence, report the string 
   // using its position in the array.
   //
   // Implemented using read_char(): simplistic but functional.
   // The timeout is a per-character timeout in milliseconds.
   // 
   // returns: the array index of the string that was encountered
   // Note: invalidates previous timeout settings of the COM port
   //
   int expect(char** candidates, unsigned long timeout);
   
   // Clears the RX and TX FIFO's
   void purge();

   // Explicit RTS and DTR handling
   // Returns: 
   //     0 = OK
   //     1 = ERROR
   int set_rts();
   int clear_rts();
   int set_dtr();
   int clear_dtr();

   string name;
	HANDLE handle;
private:
   int escape_function(unsigned long func);
};

#endif // _MY_SERIAL_H
